@@ -9,6 +9,7 @@ package health
 import (
 	"errors"
 	"fmt"
+	"os"
 	"sort"
 	"sync"
 	"sync/atomic"
@@ -32,12 +33,14 @@ var (
 	lastStreamedMapResponse time.Time
 	derpHomeRegion          int
 	derpRegionConnected     = map[int]bool{}
+	derpRegionHealthProblem = map[int]string{}
 	derpRegionLastFrame     = map[int]time.Time{}
 	lastMapRequestHeard     time.Time // time we got a 200 from control for a MapRequest
 	ipnState                string
 	ipnWantRunning          bool
 	anyInterfaceUp          = true // until told otherwise
 	udp4Unbound             bool
+	controlHealth           []string
 )
 
 // Subsystem is the name of a subsystem whose health can be monitored.
@@ -139,6 +142,13 @@ func setLocked(key Subsystem, err error) {
 	}
 }
 
+func SetControlHealth(problems []string) {
+	mu.Lock()
+	defer mu.Unlock()
+	controlHealth = problems
+	selfCheckLocked()
+}
+
 // GotStreamedMapResponse notes that we got a tailcfg.MapResponse
 // message in streaming mode, even if it's just a keep-alive message.
 func GotStreamedMapResponse() {
@@ -191,6 +201,19 @@ func SetDERPRegionConnectedState(region int, connected bool) {
 	selfCheckLocked()
 }
 
+// SetDERPRegionHealth sets or clears any problem associated with the
+// provided DERP region.
+func SetDERPRegionHealth(region int, problem string) {
+	mu.Lock()
+	defer mu.Unlock()
+	if problem == "" {
+		delete(derpRegionHealthProblem, region)
+	} else {
+		derpRegionHealthProblem[region] = problem
+	}
+	selfCheckLocked()
+}
+
 func NoteDERPRegionReceivedFrame(region int) {
 	mu.Lock()
 	defer mu.Unlock()
@@ -241,6 +264,18 @@ func selfCheckLocked() {
 	setLocked(SysOverall, overallErrorLocked())
 }
 
+// OverallError returns a summary of the health state.
+//
+// If there are multiple problems, the error will be of type
+// multierror.MultipleErrors.
+func OverallError() error {
+	mu.Lock()
+	defer mu.Unlock()
+	return overallErrorLocked()
+}
+
+var fakeErrForTesting = os.Getenv("TS_DEBUG_FAKE_HEALTH_ERROR")
+
 func overallErrorLocked() error {
 	if !anyInterfaceUp {
 		return errors.New("network down")
@@ -288,6 +323,15 @@ func overallErrorLocked() error {
 		}
 		errs = append(errs, fmt.Errorf("%v: %w", sys, err))
 	}
+	for regionID, problem := range derpRegionHealthProblem {
+		errs = append(errs, fmt.Errorf("derp%d: %v", regionID, problem))
+	}
+	for _, s := range controlHealth {
+		errs = append(errs, errors.New(s))
+	}
+	if e := fakeErrForTesting; len(errs) == 0 && e != "" {
+		return errors.New(e)
+	}
 	sort.Slice(errs, func(i, j int) bool {
 		// Not super efficient (stringifying these in a sort), but probably max 2 or 3 items.
 		return errs[i].Error() < errs[j].Error()
