@@ -637,13 +637,23 @@ func Run(ctx context.Context, logf logger.Logf, logid string, getEngine func() (
 
 	var store ipn.StateStore
 	if opts.StatePath != "" {
+		const kubePrefix = "kube:"
 		path := opts.StatePath
-		if runtime.GOOS == "windows" {
-			path = tryWindowsAppDataMigration(logf, path)
-		}
-		store, err = ipn.NewFileStore(path)
-		if err != nil {
-			return fmt.Errorf("ipn.NewFileStore(%q): %v", path, err)
+		switch {
+		case strings.HasPrefix(path, kubePrefix):
+			secretName := strings.TrimPrefix(path, kubePrefix)
+			store, err = ipn.NewKubeStore(secretName)
+			if err != nil {
+				return fmt.Errorf("ipn.NewKubeStore(%q): %v", secretName, err)
+			}
+		default:
+			if runtime.GOOS == "windows" {
+				path = tryWindowsAppDataMigration(logf, path)
+			}
+			store, err = ipn.NewFileStore(path)
+			if err != nil {
+				return fmt.Errorf("ipn.NewFileStore(%q): %v", path, err)
+			}
 		}
 		if opts.AutostartStateKey == "" {
 			autoStartKey, err := store.ReadState(ipn.ServerModeStartKey)
